#!/usr/bin/env rspec

require 'spec_helper'
require File.join(File.dirname(__FILE__), File.join('../..', 'util', 'puppet_agent_mgr.rb'))
require File.join(File.dirname(__FILE__), File.join('../..', 'util', 'puppet_agent_mgr', 'common.rb'))
require File.join(File.dirname(__FILE__), File.join('../..', 'util', 'puppet_agent_mgr', 'v2', 'manager.rb'))
require File.join(File.dirname(__FILE__), File.join('../..', 'util', 'puppet_agent_mgr', 'v3', 'manager.rb'))

module MCollective::Util
  describe PuppetAgentMgr do
    describe "#manager" do
      before do
        PuppetAgentMgr.stubs(:require)
      end

      it "should support puppet 2.x.x managers" do
        Puppet.expects(:version).returns("2.7.12")
        PuppetAgentMgr::V2::Manager.expects(:new)

        PuppetAgentMgr.manager
      end

      it "should support puppet 3.x.x managers" do
        Puppet.expects(:version).returns("3.0.0")
        PuppetAgentMgr::V3::Manager.expects(:new)

        PuppetAgentMgr.manager
      end

      it "should pass the supplied config file to the manager" do
        Puppet.expects(:version).returns("3.0.0")
        PuppetAgentMgr::V3::Manager.expects(:new).with("rspec", "puppet")

        PuppetAgentMgr.manager("rspec")
      end

      it "should pass the supplied service name to the manager" do
        Puppet.expects(:version).returns("3.0.0")
        PuppetAgentMgr::V3::Manager.expects(:new).with("rspec", "pe-puppet")

        PuppetAgentMgr.manager("rspec", "pe-puppet")
      end

      it "should fail with a friendly error for unsupported puppet versions" do
        Puppet.expects(:version).returns("0.22")

        expect { PuppetAgentMgr.manager }.to raise_error("Cannot manage Puppet version 0")
      end

      it "should fail with a friendly error when it cannot determine the Puppet version" do
        Puppet.expects(:version).returns("x")

        expect { PuppetAgentMgr.manager }.to raise_error("Cannot determine the Puppet major version")
      end
    end
  end
end
